/*********************************************************************
 *
 *      Microchip LoRa Mote Example Application
 *
 *********************************************************************
 * FileName:        SSD1306oLED.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18 USB microcontroller
 * Compiler:        XC8 v1.35+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") for its PICmicro(R) Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * File version         Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * 1.0                  04/01/16    Original.

 ********************************************************************/
/* ------------------------------------------------------------ */
/*		Include File Definitions						*/
/* ------------------------------------------------------------ */

#include <xc.h>
#include <stdint.h>
#include <stdlib.h>

#include "SSD1306oLED.h"
#include "spi.h"
#include "mcc.h"

#define oledRESET           LCD_RESET_LAT
#define oledCS              LCD_CS_LAT
#define oledDC              LCD_DC_LAT
#define oledSCK_LAT         LATBbits.LATB1
#define oledSDO_LAT         LATBbits.LATB3

#define oledRESET_TRIS      LCD_RESET_TRIS
#define oledCS_TRIS         LCD_CS_TRIS
#define oledDC_TRIS         LCD_DC_TRIS
#define oledSCK_TRIS        TRISBbits.TRISB1
#define oledSDO_TRIS        TRISBbits.TRISB3

#define csActive            0
#define csInactive          1

#define oledCommand         0
#define oledData            1

#define oledCommsWrite(data)    SPI_Exchange8bit(data)

#define Contrast_level  0x7F

#define Start_column	0x00
#define Start_page      0x00
#define	StartLine_set	0x00

#define INDEX_MAX 16
/* ------------------------------------------------------------ */
/*				Local Type Definitions		*/
/* ------------------------------------------------------------ */
const uint8_t OledFont0[] = {
#if defined(DEAD)
	/* Remove definitions for character codes 0x00-0x1F as
	** these are map to user defined characters.
	*/
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x00, NUL
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x01, SOH
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x02, STX
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x03, ETX
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x03, EOT
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x05, ENQ
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x06, ACK
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x07, BEL
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x08, BS
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x09, HT
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x0A, LF
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x0B, VT
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x0C, FF
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x0D, CR
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x0E, SO
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x0F, SI
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x10, DLE
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x11, DC1
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x12, DC2
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x13, DC3
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x13, DC4
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x15, NAK
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x16, SYN
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x17, ETB
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x18, CAN
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x19, EM
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x1A, SUB
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x1B, ESC
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x1C, FS
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x1D, GS
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x1E, RS
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x1F, US
#endif
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,		// 0x20, space
	0x00,0x00,0x00,0x5f,0x00,0x00,0x00,0x00,		// 0x21, !
	0x00,0x00,0x03,0x00,0x03,0x00,0x00,0x00,		// 0x22, "
	0x64,0x3c,0x26,0x64,0x3c,0x26,0x24,0x00,		// 0x23, #
	0x26,0x49,0x49,0x7f,0x49,0x49,0x32,0x00,		// 0x23, $
	0x42,0x25,0x12,0x08,0x24,0x52,0x21,0x00,		// 0x25, %
	0x20,0x50,0x4e,0x55,0x22,0x58,0x28,0x00,		// 0x26, &
	0x00,0x00,0x00,0x03,0x00,0x00,0x00,0x00,		// 0x27, '
	0x00,0x00,0x1c,0x22,0x41,0x00,0x00,0x00,		// 0x28, (
	0x00,0x00,0x00,0x41,0x22,0x1c,0x00,0x00,		// 0x29, )
	0x00,0x15,0x15,0x0e,0x0e,0x15,0x15,0x00,		// 0x2A, *
	0x00,0x08,0x08,0x3e,0x08,0x08,0x00,0x00,		// 0x2B, +
	0x00,0x00,0x00,0x50,0x30,0x00,0x00,0x00,		// 0x2C, ,
	0x00,0x08,0x08,0x08,0x08,0x08,0x00,0x00,		// 0x2D, -
	0x00,0x00,0x00,0x40,0x00,0x00,0x00,0x00,		// 0x2E, .
	0x40,0x20,0x10,0x08,0x04,0x02,0x01,0x00,		// 0x2F, /
	0x00,0x3e,0x41,0x41,0x41,0x3e,0x00,0x00,		// 0x30, 0
	0x00,0x00,0x41,0x7f,0x40,0x00,0x00,0x00,		// 0x31, 1
	0x00,0x42,0x61,0x51,0x49,0x6e,0x00,0x00,		// 0x32, 2
	0x00,0x22,0x41,0x49,0x49,0x36,0x00,0x00,		// 0x33, 3
	0x00,0x18,0x14,0x12,0x7f,0x10,0x00,0x00,		// 0x33, 4
	0x00,0x27,0x49,0x49,0x49,0x71,0x00,0x00,		// 0x35, 5
	0x00,0x3c,0x4a,0x49,0x48,0x70,0x00,0x00,		// 0x36, 6
	0x00,0x43,0x21,0x11,0x0d,0x03,0x00,0x00,		// 0x37, 7
	0x00,0x36,0x49,0x49,0x49,0x36,0x00,0x00,		// 0x38, 8
	0x00,0x06,0x09,0x49,0x29,0x1e,0x00,0x00,		// 0x39, 9
	0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x00,		// 0x3A, :
	0x00,0x00,0x00,0x52,0x30,0x00,0x00,0x00,		// 0x3B, //
	0x00,0x00,0x08,0x14,0x14,0x22,0x00,0x00,		// 0x3C, <
	0x00,0x14,0x14,0x14,0x14,0x14,0x14,0x00,		// 0x3D, =
	0x00,0x00,0x22,0x14,0x14,0x08,0x00,0x00,		// 0x3E, >
	0x00,0x02,0x01,0x59,0x05,0x02,0x00,0x00,		// 0x3F, ?
	0x3e,0x41,0x5d,0x55,0x4d,0x51,0x2e,0x00,		// 0x40, @
	0x40,0x7c,0x4a,0x09,0x4a,0x7c,0x40,0x00,		// 0x41, A
	0x41,0x7f,0x49,0x49,0x49,0x49,0x36,0x00,		// 0x42, B
	0x1c,0x22,0x41,0x41,0x41,0x41,0x22,0x00,		// 0x43, C
	0x41,0x7f,0x41,0x41,0x41,0x22,0x1c,0x00,		// 0x44, D
	0x41,0x7f,0x49,0x49,0x5d,0x41,0x63,0x00,		// 0x45, E
	0x41,0x7f,0x49,0x09,0x1d,0x01,0x03,0x00,		// 0x46, F
	0x1c,0x22,0x41,0x49,0x49,0x3a,0x08,0x00,		// 0x47, G
	0x41,0x7f,0x08,0x08,0x08,0x7f,0x41,0x00,		// 0x48, H
	0x00,0x41,0x41,0x7F,0x41,0x41,0x00,0x00,		// 0x49, I
	0x30,0x40,0x41,0x41,0x3F,0x01,0x01,0x00,		// 0x4A, J
	0x41,0x7f,0x08,0x0c,0x12,0x61,0x41,0x00,		// 0x4B, K
	0x41,0x7f,0x41,0x40,0x40,0x40,0x60,0x00,		// 0x4C, L
	0x41,0x7f,0x42,0x0c,0x42,0x7f,0x41,0x00,		// 0x4D, M
	0x41,0x7f,0x42,0x0c,0x11,0x7f,0x01,0x00,		// 0x4E, N
	0x1c,0x22,0x41,0x41,0x41,0x22,0x1c,0x00,		// 0x4F, O
	0x41,0x7f,0x49,0x09,0x09,0x09,0x06,0x00,		// 0x50, P
	0x0c,0x12,0x21,0x21,0x61,0x52,0x4c,0x00,		// 0x51, Q
	0x41,0x7f,0x09,0x09,0x19,0x69,0x46,0x00,		// 0x52, R
	0x66,0x49,0x49,0x49,0x49,0x49,0x33,0x00,		// 0x53, S
	0x03,0x01,0x41,0x7f,0x41,0x01,0x03,0x00,		// 0x54, T
	0x01,0x3f,0x41,0x40,0x41,0x3f,0x01,0x00,		// 0x55, U
	0x01,0x0f,0x31,0x40,0x31,0x0f,0x01,0x00,		// 0x56, V
	0x01,0x1f,0x61,0x14,0x61,0x1f,0x01,0x00,		// 0x57, W
	0x41,0x41,0x36,0x08,0x36,0x41,0x41,0x00,		// 0x58, X
	0x01,0x03,0x44,0x78,0x44,0x03,0x01,0x00,		// 0x59, Y
	0x43,0x61,0x51,0x49,0x45,0x43,0x61,0x00,		// 0x5A, Z
	0x00,0x00,0x7f,0x41,0x41,0x00,0x00,0x00,		// 0x5B, [
	0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x00,		// 0x5C,
	0x00,0x00,0x41,0x41,0x7f,0x00,0x00,0x00,		// 0x5D, ]
	0x00,0x04,0x02,0x01,0x01,0x02,0x04,0x00,		// 0x5E, ^
	0x00,0x40,0x40,0x40,0x40,0x40,0x40,0x00,		// 0x5F, _
	0x00,0x01,0x02,0x00,0x00,0x00,0x00,0x00,		// 0x60, `
	0x00,0x34,0x4a,0x4a,0x4a,0x3c,0x40,0x00,		// 0x61, a
	0x00,0x41,0x3f,0x48,0x48,0x48,0x30,0x00,		// 0x62. b
	0x00,0x3c,0x42,0x42,0x42,0x24,0x00,0x00,		// 0x63, c
	0x00,0x30,0x48,0x48,0x49,0x3f,0x40,0x00,		// 0x63, d
	0x00,0x3c,0x4a,0x4a,0x4a,0x2c,0x00,0x00,		// 0x65, e
	0x00,0x00,0x48,0x7e,0x49,0x09,0x00,0x00,		// 0x66, f
	0x00,0x26,0x49,0x49,0x49,0x3f,0x01,0x00,		// 0x67, g
	0x41,0x7f,0x48,0x04,0x44,0x78,0x40,0x00,		// 0x68, h
	0x00,0x00,0x44,0x7d,0x40,0x00,0x00,0x00,		// 0x69, i
	0x00,0x00,0x40,0x44,0x3d,0x00,0x00,0x00,		// 0x6A, j
	0x41,0x7f,0x10,0x18,0x24,0x42,0x42,0x00,		// 0x6B, k
	0x00,0x40,0x41,0x7f,0x40,0x40,0x00,0x00,		// 0x6C, l
	0x42,0x7e,0x02,0x7c,0x02,0x7e,0x40,0x00,		// 0x6D, m
	0x42,0x7e,0x44,0x02,0x42,0x7c,0x40,0x00,		// 0x6E, n
	0x00,0x3c,0x42,0x42,0x42,0x3c,0x00,0x00,		// 0x6F, o
	0x00,0x41,0x7f,0x49,0x09,0x09,0x06,0x00,		// 0x70, p
	0x00,0x06,0x09,0x09,0x49,0x7f,0x41,0x00,		// 0x71, q
	0x00,0x42,0x7e,0x44,0x02,0x02,0x04,0x00,		// 0x72, r
	0x00,0x64,0x4a,0x4a,0x4a,0x36,0x00,0x00,		// 0x73, s
	0x00,0x04,0x3f,0x44,0x44,0x20,0x00,0x00,		// 0x73, t
	0x00,0x02,0x3e,0x40,0x40,0x22,0x7e,0x40,		// 0x75, u
	0x02,0x0e,0x32,0x40,0x32,0x0e,0x02,0x00,		// 0x76, v
	0x02,0x1e,0x62,0x18,0x62,0x1e,0x02,0x00,		// 0x77, w
	0x42,0x62,0x14,0x08,0x14,0x62,0x42,0x00,		// 0x78, x
	0x01,0x43,0x45,0x38,0x05,0x03,0x01,0x00,		// 0x79, y
	0x00,0x46,0x62,0x52,0x4a,0x46,0x62,0x00,		// 0x7A, z
	0x00,0x00,0x08,0x36,0x41,0x00,0x00,0x00,		// 0x7B, {
	0x00,0x00,0x00,0x7f,0x00,0x00,0x00,0x00,		// 0x7C, |
	0x00,0x00,0x00,0x41,0x36,0x08,0x00,0x00,		// 0x7D, }
	0x00,0x18,0x08,0x08,0x10,0x10,0x18,0x00,		// 0x7E, ~
	0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55			// 0x7F, DEL
};

const uint8_t *selectedFont = OledFont0;
/* ------------------------------------------------------------ */
/*		Private Function Declarations		        */
/* ------------------------------------------------------------ */
void Set_Page_Address(uint8_t);
void Set_Column_Address(uint8_t);
void oled_devInit(void);
void oled_writeCommand(uint8_t);
void oled_writeData(uint8_t);
void oled_setContrastControlRegister(uint8_t);

/* ------------------------------------------------------------ */
/*				Procedure Definitions		*/
/* ------------------------------------------------------------ */
/****************SET PAGE ADDRESS**************/
// 0~15
void Set_Page_Address(uint8_t address)
{
    address = 0xB0 | address;
    oledCS = csActive;
    oled_writeCommand(address);
    oledCS = csInactive;
}

/****************SET COLUMN ADDRESS**************/
void Set_Column_Address(uint8_t address)
{
    oledCS = csActive;
    oled_writeCommand(0x10 | (address >> 4) );
    oled_writeCommand(0x0F & address);
    oledCS = csInactive;
}

void oled_setFont(const uint8_t *font)
{
    if(font!=0)
    {
        selectedFont=font;
    }
}

void oled_putUint8(uint8_t number, uint8_t station_dot, uint8_t start_page)
{
    display_data_t numData;
    numData.d2 = '0' + (number % 10);
    number /= 10;
    numData.d1 = '0' + (number % 10);
    number /= 10;
    numData.d0 = '0' + (number % 10);
    
    numData.b[3]=0; // Terminate the string

    oled_putString(numData.b,station_dot,start_page);
}

void oled_putUint16(uint16_t number, uint8_t station_dot, uint8_t start_page)
{
    display_data_t numData;
    numData.d4 = '0' + (number % 10);
    number /= 10;
    numData.d3 = '0' + (number % 10);
    number /= 10;
    numData.d2 = '0' + (number % 10);
    number /= 10;
    numData.d1 = '0' + (number % 10);
    number /= 10;
    numData.d0 = '0' + (number % 10);

    numData.b[5]=0; // Terminate the string

    oled_putString(numData.b,station_dot,start_page);
}

void oled_putChar(char k,uint8_t station_dot, uint8_t start_page)
{
    uint8_t index;

    Set_Page_Address(start_page);
    Set_Column_Address(Start_column + station_dot * 8);
    
    oledCS = csActive;
    for(index = 0; index < 8; index++)
    {
        oled_writeData(*(selectedFont + 8 * (k - 0x20) + index));
    }
    oledCS = csInactive;
}

void oled_putString(const char *string, uint8_t station_dot, uint8_t start_page)
{
    uint8_t index;
    for(index = station_dot; index < INDEX_MAX; index++)
    {
        if (*string != 0)
        {
            oled_putChar(*string, index, start_page);
            *string++;
        }
        else
        {
            break;
        }
    }
}
/****************SHW DOT******************/
void oled_clear(void)
{
    oled_clearRow(0);
    oled_clearRow(1);
    oled_clearRow(2);
    oled_clearRow(3);
}

void oled_clearRow(uint8_t page)
{
    uint8_t index;

    for(index = 0; index < INDEX_MAX; index++)
    {
      oled_putChar( ' ', index, page);
    }
}

void oled_init(void)
{
    // Set up I/O pins
    oledRESET_TRIS=0;
    oledCS_TRIS=0;
    oledDC_TRIS=0;

    oled_devInit();          //Init the OLED display hardware.
    oled_setDisplay(Normal);
    oled_setFont(OledFont0);
    oled_clear();
}

void oled_devInit(void)
{
    oledCS = csInactive;
    
    oledDC = oledData;
    /* Start by turning VDD on and wait a while for the power to come up.    */

    /******************RESET********************/
    oledRESET = 1;
    __delay_us(20);
    oledRESET = 0;
    __delay_us(200);
    oledRESET = 1;

    oledCS = csActive;
    __delay_us(200);
    oledDC = oledCommand;
    __delay_us(200);

    // Init sequence for SPI OLED module
    oled_writeCommand(0xAE);       // Display Off

    oled_writeCommand(0xD5);       // set display clock divide ratio/oscillator frequency
    oled_writeCommand(0x80);       // set divide ratio

    oled_writeCommand(0xA8);       // set multiplex ratio(1 to 64)
    oled_writeCommand(0x1F);       // 1/32 duty

    oled_writeCommand(0xD3);       // set display offset
    oled_writeCommand(0x00);       // not offset

    oled_writeCommand(0x8D);       // set Charge Pump enable/disable
    oled_writeCommand(0x14);       // set(0x10) disable

    oled_writeCommand(0x40);       // set start line address

    oled_writeCommand(0xA6);       // set normal display

    oled_writeCommand(0xA4);       // Disable Entire Display On

    oled_writeCommand(0xA0);      // set segment re-map 128 to 0

    oled_writeCommand(0xC0);       // Set COM Output Scan Direction 64 to 0

    oled_writeCommand(0xDA);       // set com pins hardware configuration
    oled_writeCommand(0x42);       //

    oled_writeCommand(0x81);       // set contrast control register
    oled_writeCommand(Contrast_level);

    oled_writeCommand(0xD9);       // set pre-charge period
    oled_writeCommand(0xF1);

    oled_writeCommand(0xDB);       // set vcomh
    oled_writeCommand(0x40);

    oled_writeCommand(0xAF);       // Display On
    oledCS = csInactive;
}


void oled_setDisplay (oLED_DisplayMode_t displayType)
{
    displayType =  0xA6 + displayType;

    oledCS = csActive;

    oled_writeCommand(displayType);       // Normal [0] / Reverse [1]
    
    oledCS = csInactive;
}

void oled_displayOff (bool state)
{
    oledCS = csActive;
    oled_writeCommand(0xAF - state);       // Display On
    oledCS = csInactive;
}

void oled_writeCommand(uint8_t command)
{
    oledDC = oledCommand;
    oledCommsWrite(command);
}

void oled_writeData(uint8_t data)
{
    oledDC = oledData;
    oledCommsWrite(data);
}

void oled_setContrastControlRegister(uint8_t mod)
{
    oledCS = csActive;

    oled_writeCommand(0x81);
    oled_writeCommand(mod);

    oledCS = csInactive;
}



